import { Component, Input, OnChanges, SimpleChanges, ViewChildren, QueryList } from '@angular/core';

import { of } from 'rxjs';
import { map, catchError, take } from 'rxjs/operators';

import { Host } from '@overflow/model/discovery';
import { PingResult } from '@overflow/model/ping';

import { AutoHeightDirective } from '@overflow/commons/ui/directive/auto-height.directive';
import { PingService } from '../../../service/ping.service';
import { PingOption } from '@overflow/model/config/ping';

import * as log from '../../../core/util/log';
import { MetaDiscovererTypeEnum, toMetaDiscovererTypeEnum } from '@overflow/model/meta';

@Component({
  selector: 'app-infra-detail-host',
  templateUrl: './host.component.html',
  styleUrls: ['./host.component.scss'],
})
export class HostComponent implements OnChanges {
  _host: Host;

  @ViewChildren(AutoHeightDirective) autoHeightDirectives: QueryList<AutoHeightDirective>;


  pingWaiting: boolean;

  count: number;
  interval: number;
  deadline: number;

  pingResult: PingResult;
  pingResultRaw: string;

  securityMap: Map<string, string>;

  @Input()
  set host(host: Host) {
    this._host = host;

    if (undefined !== this._host.discoveredBy) {
      this._host.discoveredBy.forEach((discovererType) => {
        const discovererTypeEnum = toMetaDiscovererTypeEnum(discovererType);
        switch (discovererTypeEnum) {
          case MetaDiscovererTypeEnum.ICMP:
            this.securityMap.set(MetaDiscovererTypeEnum.ICMP, 'ICMP(Ping) is not safe.');
            break;
          default:
            break;
        }
      });
    }
  }
  get host(): Host {
    return this._host;
  }

  constructor(
    private pingService: PingService,
  ) {
    this.pingWaiting = false;
    this.count = 5;
    this.interval = 1;
    this.deadline = 1;

    this.securityMap = new Map();
  }

  ngOnChanges(simpleChanges: SimpleChanges): void {
    this.pingResult = null;
  }


  doPing() {
    this.pingWaiting = true;

    const option: PingOption = {
      count: this.count,
      interval: this.interval,
      deadline: this.deadline,
    };

    const _host: Host = {
      ...this.host,
      portList: null,
    };

    this.pingService.pingHost(_host, option)
      .pipe(
        map((pingResult: PingResult) => {
          if (pingResult) {
            this.pingResult = pingResult;
            this.pingResultRaw = pingResult.raw.join('\r\n');
          }
          this.pingWaiting = false;
        }),
        catchError(error => {
          log.error(error);
          alert('An error has occurred.');
          this.pingWaiting = false;
          return of();
        }),
        take(1)
      )
      .subscribe();
  }

  onTabViewChange($event) {
    const index = String($event.index);
    this.autoHeightDirectives.map(item => {
      if (item.key === index) {
        item.forceCalculate();
      }
    });
  }

}
